/*-----------------------------------------------------------------------------
 * Snow Globe App
 * SJSU Student 
 * CS 185c Homework 4
 * 11/24/2010
 * Some code samples found in microsoft's create app hub.
 * Tilt / Perspective sample found on create.msdn.com
*-----------------------------------------------------------------------------
*/
using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.GamerServices;

#if WINDOWS_PHONE
using System.Device.Location;
#endif

namespace CS185HW4
{
    public class SnowGlobeApp : Microsoft.Xna.Framework.Game
    {
        #region Fields
        // distance from eye to screen, in pixel units
        const float eyeDistance = 2000;
        
        // distance from eye to near clip plane, as a fraction of the Z distance to the screen. We do it this way
        // because we can end up with some highly skewed projection matrices if the devices is tilted enough.
        const float nearPlane = 0.5f;

        // distance from the eye to the far clip plane, in pixel units
        const float farPlaneDistance = 4000;

        // size of the box that everything takes place in, measured in pixel units
        BoundingBox worldBox = new BoundingBox
        {
            Min = new Vector3(-500, -600, -500),
            Max = new Vector3(500, 600, 0)
        };
       

        // "down" direction (smoothed accelerometer reading) to use as our reference position. The user
        // can reset this by touching the screen.
        Vector3 referenceDown = -Vector3.UnitZ;
        
        GraphicsDeviceManager graphics;
        AccelerometerHelper accelerometer;
        SnowGlobeSimulation simulation;

        
        SpriteBatch batch;
        SpriteFont arialFont;
        Texture2D boxTexture;
        Texture2D snowTexture;
        #endregion
        
        #region Initialization
        public SnowGlobeApp()
        {
            Content.RootDirectory = "Content";
            graphics = new GraphicsDeviceManager(this);

            // Request portrait mode
            graphics.PreferredBackBufferWidth = 480;
            graphics.PreferredBackBufferHeight = 800;

            // Turn off all phone UI
          //  graphics.IsFullScreen = true;
            Guide.IsScreenSaverEnabled = false;

            TouchPanel.EnabledGestures =  GestureType.Tap| GestureType.Flick;

            // Frame rate is 30 fps by default
            IsFixedTimeStep = true;
            TargetElapsedTime = TimeSpan.FromTicks(333333);

            accelerometer = new AccelerometerHelper(this);
          //  accelerometer.SimulateMovement = true;
            Components.Add(accelerometer);

            simulation = new  SnowGlobeSimulation(this);
            simulation.AddWalls(worldBox);
            simulation.AddSnow(200, worldBox);
            Components.Add(simulation);
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            base.Initialize();

#if WINDOWS_PHONE
            geoWatcher = new GeoCoordinateWatcher(GeoPositionAccuracy.High);
            geoWatcher.StatusChanged += new EventHandler<GeoPositionStatusChangedEventArgs>(geoWatcher_StatusChanged);
            geoWatcher.PositionChanged += new EventHandler<GeoPositionChangedEventArgs<GeoCoordinate>>(geoWatcher_PositionChanged);
            geoWatcher.Start(true);
#endif
        }

        private string _lastGPSStatus = "";
        private string _lastGPSPosition = "";
#if WINDOWS_PHONE
        GeoCoordinateWatcher geoWatcher;
        void geoWatcher_PositionChanged(object sender, GeoPositionChangedEventArgs<GeoCoordinate> e)
        {
            _lastGPSPosition = e.Position.Location.ToString();
        }

        void geoWatcher_StatusChanged(object sender, GeoPositionStatusChangedEventArgs e)
        {
            _lastGPSStatus = e.Status.ToString();
        }
#endif
        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            //background image
            boxTexture = Content.Load<Texture2D>("snow");
            //snowflake image
            snowTexture = Content.Load<Texture2D>("snowflake2");
            arialFont = Content.Load<SpriteFont>("arial");
            batch = new SpriteBatch(this.GraphicsDevice);
            simulation._snowTexture = snowTexture;
            simulation.LoadContent(this.GraphicsDevice);
          //  worldGeometry = DebugDraw.CreateBoxInterior(GraphicsDevice, worldBox);
            
        }
        #endregion

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            accelerometer.Update(gameTime);

            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            KeyboardState kb = Keyboard.GetState();
           
           
            if (kb.IsKeyDown(Keys.Down))
                simulation._snowmanLeftEye.Position.Y -= 10;
            if (kb.IsKeyDown(Keys.Up))
                simulation._snowmanLeftEye.Position.Y += 10;
            if (kb.IsKeyDown(Keys.Left))
                simulation._snowmanLeftEye.Position.Z -= 10;
            if (kb.IsKeyDown(Keys.Right))
                simulation._snowmanLeftEye.Position.Z += 10;
           
            
            Vector2 _lastflickVector;
            //look for a guesture, we are going to show the gps coordinates if they tap, and record the swipe vector if they swipe...
            if (TouchPanel.IsGestureAvailable )
            {
               var gs=   TouchPanel.ReadGesture();
               if ((gs.GestureType & GestureType.Tap) == GestureType.Tap)
               {
                   _keepShowingGPSUntil = gameTime.TotalGameTime.TotalSeconds + 15.0;
               }
               if ((gs.GestureType & GestureType.Flick) == GestureType.Flick)
               {
                   _lastflickVector = gs.Delta;
                  // _lastflickVector.Normalize();
                   simulation.Swiped = true;
                   simulation.LastSwipeDirection = new Vector3(_lastflickVector.X,_lastflickVector.Y * -1, 0.0f);
               }
         //       referenceDown = Vector3.Normalize(accelerometer.SmoothAcceleration);
            }
          
            base.Update(gameTime);
        }

        private double _keepShowingGPSUntil;


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            
            const float zThreshold = 0.4f;

            GraphicsDevice.Clear(Color.CornflowerBlue);

            batch.Begin();
            batch.Draw(boxTexture,new Vector2(0,0),Color.White);
            batch.End();
            // Make the light always come from the actual ceiling
            Vector3 lightDirection = Vector3.Normalize(accelerometer.SmoothAcceleration);
           // lightDirection =  new Vector3(lightDirection.X,lightDirection.Z, -lightDirection.Y);
            
            Vector3 eyeDirection = ComputeEyeVector();

            if (eyeDirection.Z < zThreshold)
            {
                // Limit how far we distort the perspective
                eyeDirection.Z = zThreshold;
                eyeDirection.Normalize();
            }
            Vector3 eyePosition = eyeDirection * eyeDistance;

            Matrix world = Matrix.Identity;

            Matrix view = Matrix.CreateLookAt(
                new Vector3(eyePosition.X, eyePosition.Y, eyePosition.Z),
                new Vector3(eyePosition.X, eyePosition.Y, 0),
                new Vector3(0, 1, 0)
            );

            Matrix projection = Matrix.CreatePerspectiveOffCenter(
                (-eyePosition.X - graphics.GraphicsDevice.Viewport.Width*.5f) * nearPlane,
                (-eyePosition.X + graphics.GraphicsDevice.Viewport.Width * .5f) * nearPlane,
                (-eyePosition.Y - graphics.GraphicsDevice.Viewport.Height * .5f) * nearPlane,
                (-eyePosition.Y + graphics.GraphicsDevice.Viewport.Height * .5f) * nearPlane,
                eyePosition.Z * nearPlane,
                farPlaneDistance);

         //   worldGeometry.BasicEffect.DirectionalLight0.Direction = lightDirection;
          //  worldGeometry.Draw(ref world, ref view, ref projection, boxTexture);

            simulation.Draw(view, projection, lightDirection);

            if (_keepShowingGPSUntil > gameTime.TotalGameTime.TotalSeconds)
            {
                batch.Begin();
                batch.DrawString(arialFont, _lastGPSStatus, new Vector2(10, 10), Color.Black);
                batch.DrawString(arialFont, _lastGPSPosition, new Vector2(10, 30), Color.Black);
                batch.End();
            }

            base.Draw(gameTime);
        }

        // Compute (guess) the user's eye direction, given a reference 'down'
        // direction and the current 'down' direction.
        //
        // 'world' vectors in this function refer to the actual real-world
        // coords from the (estimated) user's perspective.
        // We don't really know where the user is relative to the screen,
        // so we have to make an assumption about how they hold and tilt
        // the device.
        Vector3 ComputeEyeVector()
        {
            float referencePitch = (float)Math.Asin(referenceDown.Y);
            float rollEpsilon = .1f;

            Vector3 worldDown = -accelerometer.SmoothAcceleration;
            worldDown.Normalize();

            Vector3 worldRight = Vector3.Cross(Vector3.UnitY, worldDown);

            if (worldRight.LengthSquared() < rollEpsilon)
            {
                // The device is held nearly vertically, so the worldRight vector isn't well-defined
                // (its length is close to zero). Just use our local right vector as the world
                // right vector, which means we generate an orientation with no roll.
                //
                worldRight = Vector3.Right;
                worldRight.Normalize();
            }
            else
            {
                // We have a good 'right' vector; normalize it for CreateFromAxisAngle() below.
                worldRight.Normalize();
            }
            Quaternion rot = Quaternion.CreateFromAxisAngle(worldRight, -referencePitch);
         //   return Vector3.Transform(worldDown, rot);
            return Vector3.Forward;
        }
    }
}
