/*-----------------------------------------------------------------------------
 * Snow Globe Simulation
 * SJSU Student
 * CS 185c Homework 4
 * 11/24/2010
 * Some code samples found in microsoft's create app hub.
 * Tilt / Perspective sample found on create.msdn.com
 * 
*-----------------------------------------------------------------------------
*/
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Content;
using System;
using System.Collections.Generic;


namespace CS185HW4
{

    
    public class Ball
    {
        public Vector3 Position;
        public Vector3 Velocity;
        public bool Fixed;
        public float Radius;
        public float Mass;
        public Color Color;
    }
     
    public class SnowFlake
    {
        public Vector3 Position;
        public Vector3 Velocity;
        public float Mass;
        public Color Color;
        public float Radius;
        public int ttl;
    }


    /// <summary>
    /// This sample shows how to draw 3D geometric collisions
    /// </summary>
    /// 
    public class SnowGlobeSimulation : GameComponent
    {
        #region Fields

        GeometricPrimitive highQualitySphere;


        SpriteBatch flakeBatch;

        List<Plane> Walls = new List<Plane>();
        List<SnowFlake > flakes = new List<SnowFlake >();

        public Ball _snowmanBase, _snowmanBody, _snowmanHead, _snowmanLeftEye, _snowmanRightEye, _snowmanNose;

         public Texture2D _snowTexture;
        

        #endregion

        #region Initialization
        public SnowGlobeSimulation(Game game)
            : base(game)
        {
            Walls = new List<Plane>();
          
        }

        public void LoadContent(GraphicsDevice gd)
        {
            flakeBatch = new SpriteBatch(gd);
        }

        public void AddWall(Plane wallPlane)
        {
            Walls.Add(wallPlane);
        }

        public void AddWalls(BoundingBox worldBox)
        {
            AddWall(new Plane(-Vector3.UnitX, worldBox.Max.X));
            AddWall(new Plane(Vector3.UnitX, -worldBox.Min.X));
            AddWall(new Plane(-Vector3.UnitY, worldBox.Max.Y));
            AddWall(new Plane(Vector3.UnitY, -worldBox.Min.Y));
        }
        int _snowFlakeRadius = 32;
        BoundingBox _simBox;
        public void ResetFlake(ref SnowFlake flake,BoundingBox  inBox)
        {
              Random rng = RandomUtil.SharedRandom;
              flake.Position = new Vector3(
                          rng.NextFloat(inBox.Min.X + _snowFlakeRadius, inBox.Max.X - _snowFlakeRadius),
                          rng.NextFloat(inBox.Min.Y + _snowFlakeRadius, inBox.Max.Y - _snowFlakeRadius),
                          rng.NextFloat(inBox.Min.Z + _snowFlakeRadius, inBox.Max.Z - _snowFlakeRadius));
              
              flake.Velocity = Vector3.Zero;
            
        }

        public void AddSnow(int flakeCount, BoundingBox inBox)
        {
            _simBox = inBox;
            Random rng = RandomUtil.SharedRandom;
           
            for (int i = 0; i < flakeCount; i++)
            {
                SnowFlake flake =  new SnowFlake()
                {
                    ttl = 0,
                    Color = Color.White,
                    Radius = _snowFlakeRadius
                    
                };

              
                flakes.Add(  flake);
            }
        }


        public override void Initialize()
        {
            base.Initialize();
           
            highQualitySphere = new SpherePrimitive(Game.GraphicsDevice,1,64);
            initSnowman();
        }

        public void initSnowman()
        {
           _snowmanBase = new Ball()
            {
                Position = new Vector3(0, -300, -200),
                Color = Color.WhiteSmoke,
                Radius=200
            };

           _snowmanBody = new Ball()
            {
                Position = new Vector3(0, -70, -150),
                Color = Color.WhiteSmoke,
                Radius = 160
            };

           _snowmanHead = new Ball()
            {
                Position = new Vector3(0, 145, -145),
                Color = Color.WhiteSmoke,
                Radius = 120
            };

           _snowmanLeftEye = new Ball()
           {
               Position = new Vector3(-30, 205, -5),
               Color = Color.LightSkyBlue,
               Radius = 10
           };
           _snowmanRightEye = new Ball()
           {
               Position = new Vector3(30, 205, -5),
               Color = Color.LightSkyBlue,
               Radius = 10
           };
           _snowmanNose = new Ball()
           {
               Position = new Vector3(0, 180, -5),
               Color = Color.LightSkyBlue,
               Radius = 10
           };



        }

        #endregion

        #region Update and Draw


        private void ResetAllSnow()
        {
            for (int i = 0; i < flakes.Count; i++)
            {
                SnowFlake flake = flakes[i];
                flake.ttl = 10;
                ResetFlake(ref flake, _simBox);
            }
        }

        public Vector3 LastSwipeDirection = Vector3.Zero;
        public bool Swiped = false;
        private bool usingAccelGravity = true;
        /// <summary>
        /// UpdateBalls()
        /// Evalute flakes physics
        /// </summary>
        protected void UpdateSimulation(float dt)
        {
            Vector3 gravity = Vector3.Zero;
            IAccelerometerService accel = (IAccelerometerService)Game.Services.GetService(typeof(IAccelerometerService));
            //if we are shaking, reset the snow and use the accelerometer for gravity
            if (accel.IsShaking)
            {
                ResetAllSnow();
                usingAccelGravity = true;
                
            }
            //if we are swiped while snowing, show the snow in that direction
            if (Swiped)
            {
                ResetAllSnow();
                Swiped = false;
                usingAccelGravity = false;
            }
            if (usingAccelGravity)
            {
                gravity = accel.RawAcceleration * 500;
            }
            else
            {
                gravity = LastSwipeDirection;
            }
       
           
          


   

            // Resolve collisions with floor
            for (int i = 0; i < flakes.Count; i++)
            {
                SnowFlake flake = flakes[i];
                if (flake.ttl>0)
                {
                    flake.Position += flake.Velocity * dt; // +(gravity * (dt * dt * 0.5f));
                    flake.Velocity += gravity * dt;

                    // Resolves collisions with walls
                    foreach (Plane wall in Walls)
                    {
                        float depth = wall.DotCoordinate(flake.Position) - flake.Radius;
                        if (depth < 0)
                        {
                            // collision with wall
                            flake.Position -= wall.Normal * depth;

                            float impact = wall.DotNormal(flake.Velocity);
                            if (impact < 0)
                            {

                                
                                flake.ttl--;
                                ResetFlake(ref flake, _simBox);

                            }
                        }
                    }
                }
            }
        }


        /// <summary>
        /// Allows the game to run logic.
        /// </summary>
        public override void Update(GameTime gameTime)
        {
            // We break each game update into multiple simulation substeps,
            // to get more accurate handling of multi-object collisions.
            const int substeps = 4;
            float dt = (float)gameTime.ElapsedGameTime.TotalSeconds;
            for (int i = 0; i < substeps; i++)
            {
                UpdateSimulation(dt / substeps);
            }
            base.Update(gameTime);
        }

        private void DrawBall(Matrix view, Matrix projection, Ball b, bool isDrawingShadow, Matrix shadow)
        {
            Matrix world = Matrix.CreateScale(b.Radius * 2.0f);
            world.Translation = b.Position;

            if (isDrawingShadow)
            {
                world *= shadow;
                highQualitySphere.Draw(world, view, projection,  Color.Black, false);
            }
            else
            {
                highQualitySphere.Draw(world, view, projection,  b.Color, false);
            }
            
        }

        private void DrawSnowman( Matrix view, Matrix projection, Vector3 lightDirection,bool isDrawingShadow,Matrix shadow)
        {

            highQualitySphere.LightDirection = lightDirection;
            DrawBall(view, projection, _snowmanBase, isDrawingShadow,shadow);
            DrawBall(view, projection, _snowmanBody, isDrawingShadow, shadow);
            DrawBall(view, projection, _snowmanHead, isDrawingShadow, shadow);
            DrawBall(view, projection, _snowmanLeftEye, isDrawingShadow, shadow);
            DrawBall(view, projection, _snowmanRightEye, isDrawingShadow, shadow);
            DrawBall(view, projection, _snowmanNose, isDrawingShadow, shadow);

            if (isDrawingShadow)
            {
             
            }
            else
            {


            }
            

        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        public void Draw(Matrix view, Matrix projection, Vector3 lightDirection)
        {
            GraphicsDevice graphicsDevice = Game.GraphicsDevice;


            highQualitySphere.LightDirection = lightDirection;

            RasterizerState tmpRasterizerState = new RasterizerState();
            tmpRasterizerState.MultiSampleAntiAlias = true;
            tmpRasterizerState.FillMode = FillMode.Solid;
            tmpRasterizerState.CullMode = CullMode.CullCounterClockwiseFace;
            
            // Apply the updated Rasterizer state
            graphicsDevice.RasterizerState = tmpRasterizerState;

           
            DrawSnowman(view, projection, lightDirection, false, view);



            flakeBatch.Begin();
            for (int i = 0; i < flakes.Count; i++)
            {
                if (flakes[i].ttl>0)
                {
                    //    Matrix world = Matrix.CreateScale(flakes[i].Radius * 2.0f);
                    //  world.Translation = flakes[i].Position;
                    //TODO:draw snowflake
                    float scale = flakes[i].Position.Z / 400;
                    //flakeBatch.Draw(_snowTexture, new Vector2(flakes[i].Position.X, flakes[i].Position.Y), null, flakes[i].Color, 1.0f, Vector2.Zero, scale , SpriteEffects.None ,0.0f);

                    Vector3 center = this.Game.GraphicsDevice.Viewport.Project(flakes[i].Position, projection,view, Matrix.Identity);

                    flakeBatch.Draw(_snowTexture, new Vector2(center.X, center.Y), flakes[i].Color);
                }
            }
            flakeBatch.End();



            // Since we know where the ground planes are, generate simple shadows
            // by squashing the geometry onto these planes and rendering it in black.
            for (int i = 0; i < Walls.Count;i++ )
            {
                const float epsilon = 0.01f;
                const float planeOffset = 0.5f; // offset shadows to avoid depth buffer artifacts
                if (Walls[i].DotNormal(lightDirection) >= -epsilon)
                    continue;

                Matrix shadowMatrix = Matrix.CreateShadow(lightDirection, new Plane(-Walls[i].Normal, planeOffset - Walls[i].D));

                for (int fi = 0; fi < flakes.Count; fi++)
                {
                    Matrix world = Matrix.CreateScale(flakes[fi].Radius * 2.0f);
                    world.Translation = flakes[fi].Position;

                    world *= shadowMatrix;
                    //TODO:draw flake shadow lowQualitySphere.Draw(world, view, projection, Color.Black, false);

                }
                DrawSnowman(view, projection, lightDirection, true, shadowMatrix);
            }

            // Reset the fill mode renderstate.
            tmpRasterizerState = new RasterizerState();
            tmpRasterizerState.FillMode = FillMode.Solid;
            tmpRasterizerState.CullMode = CullMode.CullCounterClockwiseFace;
            graphicsDevice.RasterizerState = tmpRasterizerState;
        }


        #endregion
    }
}
    
